;+
; NAME: rbsp_efw_cal_spec
;
; PURPOSE: Calibrate RBSP spectral data
;
; NOTES: Intended to be called from rbsp_load_efw_spec.pro
;
; TO DO: 
;		-No accounting for midday source change
;		-only set up for 64 bins at the moment
;		-not using boom shorting factor
;
;		There are 7 returned spec channels. The possibilities are
;			Select 7 of: E12dc,E34dc,E56dc
;						 E12ac,E34ac,E56ac
;						 Edcpar,Edcprp
;						 Eacpar,Eacprp
;						 V1ac,V2ac,V3ac,V4ac,V5ac,V6ac
;						 SCMU,SCMV,SCMW
;						 SCMpar,SCMprp,
;						 (V1dc+V2dc+V3dc+V4dc)/4,
;						 Edcprp2, Eacprp2, SCMprp2
;		
;
;
; Calibration procedure
;
;	subtract offset
;		vals = (double(vals) - offset[j])
;   Compensate for 4 bit dynamic range compression
;       vals /= 16.
;	Divide the 0th bin by 4 (from weirdness doc)
;		vals[*,0] /= 4.
;	Compensate for using Hanning window
;		vals /= 0.375^2
;	Apply ADC conversion
;		vals *= adc_factor^2
;	Apply channel gain
;		vals *= gain[j]^2   (V^2/Hz at this point)
;	Apply boom length to change from V^2/Hz to mV/m^2/Hz				
;		vals *=  conversion[j]^2   ;mV/m ^2 /Hz
;
;	Apply freq dependent gain
;			
;
;
;
; HISTORY:
;   2012-09-xx: Created by Aaron W Breneman, University of Minnesota
;	2013-04-xx: added automated channel selection
;	2013-04-23: added freq-dependent gain
;
; VERSION:
; $LastChangedBy:  $
; $LastChangedDate:  $
; $LastChangedRevision:  $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/missions/rbsp/efw/rbsp_efw_cal_fbk.pro $
;
;-


pro rbsp_efw_cal_spec, probe=probe, trange=trange, datatype=datatype, pT=pT


;Spec sources
get_data,'rbsp'+probe+'_efw_spec_64_ccsds_data_DFB_config',data=dfbc
dfbconfig = rbsp_get_efw_dfb_config(dfbc.y)
src = dfbconfig[0].spec_config.spec_src



;***********GET THE CALIBRATION FILE*****************
;path = '~/Desktop/code/Aaron/RBSP/TDAS_trunk_svn/ssl_general/missions/rbsp/efw/calibration_files/'
;restore,path + 'gain_curves.idl'
   
;plot,tmp.freqs,tmp.gain


;
	;rbsp_efw_init
;
;	rbsp_burst=!rbsp_efw
;	rbsp_burst.remote_data_dir='http://tetra.space.umn.edu/rbspdata/'
;	
;	dt=1.e-6
;	
;	if keyword_set(probe) then p_var=probe else p_var='*'
;	vprobes = ['a','b']
;	p_var = strfilter(vprobes, p_var ,delimiter=' ',/string)
;	
;	for p=0,size(p_var,/n_elements)-1 do begin
;	
;		rbspx = 'rbsp'+ p_var[p]
;	
;		for b=1,2 do begin
;	
;			bid='vb'+string(b,format='(I0)')
;		
;			format = rbspx + '/'+bid+'_playback/YYYY/'+ $
;					rbspx+'_efw_'+bid+'_playback_YYYYMMDD_v*.txt'
;
;			relpathnames = file_dailynames(file_format=format,trange=trange, $
;					addmaster=addmaster)
;
;			files=file_retrieve(relpathnames,/last_version,_extra=rbsp_burst)
;

;***********GET THE CALIBRATION FILE*****************





if datatype eq 'spec' then begin

	compile_opt idl2
	
	;dprint,verbose=verbose,dlevel=4,'$Id: rbsp_efw_cal_filterbank.pro 10920 2012-09-17 17:08:00Z aaron_breneman $'
	
	if ~keyword_set(trange) then trange = timerange()
	
	cp0 = rbsp_efw_get_cal_params(trange[0])
	
	
	; first see if we have 36, 64, or 112 bins
	names=''
	nn=''
	names=tnames('rbsp'+probe+'*spec?')
	if strlen(names[0]) ne 0 then nn=strmid(names[0],10,1)

	case nn of
		'3': bins = '36'
		'6': bins = '64'
		'1': bins = '112'
		else: dprint,'NOT CORRECT NUMBER OF BINS...CALIBRATION ABORTED**************'
	endcase

	case strlowcase(probe) of
	  'a': cp = cp0.a
	  'b': cp = cp0.b
	  else: dprint, 'INVALID PROBE NAME. CALIBRATION ABORTED*************'
	endcase
	

	adc_factor = 2.5d / 32767.5d
	
;	boom_shorting_factor = cp.boom_shorting_factor
	
	rbspx = 'rbsp' + probe[0]
	
	
	tvar = rbspx + ['_efw_64_spec0',$
					'_efw_64_spec1',$
					'_efw_64_spec2',$
					'_efw_64_spec3',$
					'_efw_64_spec4',$
					'_efw_64_spec5',$
					'_efw_64_spec6']


	gain = replicate(0.,n_elements(src))
;	gain_64 = replicate(0.,64)
;	gain_64 = [[gain_64],[gain_64],[gain_64],[gain_64],[gain_64],[gain_64],[gain_64]]
	offset = replicate(0.,n_elements(src))
	conversion = replicate(0.,n_elements(src))
	units = replicate('',n_elements(src))
	zlims = replicate(0d,n_elements(src),2)


	if ~keyword_set(pT) then pTf = 1. else pTf = 1000.
	if ~keyword_set(pT) then pTnT = 'nT' else pTnT = 'pT'
	if ~keyword_set(pT) then pTnT_zscale = 1. else pTnT_zscale=1d3

	;Define freq-dependent gain arrays
	
g36_e12 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999986,0.999980,0.999974,0.999962,$
		0.999943,0.999921,0.999894,0.999848,0.999772,0.999682,0.999577,0.999391,0.999090,$
		0.998729,0.998308,0.997565,0.996364,0.994924,0.993246,0.990285,0.985512,0.979806,$
		0.973179,0.961540,0.942937,0.920945,0.895733,0.852296,0.784991,0.708626,0.625301]
g36_e34 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999985,0.999979,0.999973,0.999960,$
		0.999941,0.999918,0.999890,0.999842,0.999764,0.999670,0.999561,0.999368,0.999056,$
		0.998682,0.998245,0.997474,0.996228,0.994735,0.992994,0.989923,0.984973,0.979057,$
		0.972186,0.960123,0.940850,0.918077,0.891987,0.847079,0.777596,0.698919,0.613321]
g36_e56 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999985,0.999979,0.999972,0.999959,$
		0.999939,0.999914,0.999886,0.999836,0.999755,0.999658,0.999544,0.999344,0.999020,$
		0.998632,0.998178,0.997378,0.996085,0.994534,0.992728,0.989540,0.984404,0.978265,$
		0.971138,0.958627,0.938646,0.915052,0.888039,0.841586,0.769823,0.688741,0.600811]

g64_e12 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999986,0.999980,0.999974,0.999966,$
		0.999958,0.999948,0.999938,0.999927,0.999914,0.999901,0.999887,0.999864,0.999830,$
		0.999793,0.999751,0.999706,0.999657,0.999605,0.999548,0.999457,0.999321,0.999171,$
		0.999005,0.998825,0.998630,0.998419,0.998194,0.997828,0.997287,0.996687,0.996026,$
		0.995306,0.994527,0.993688,0.992790,0.991331,0.989179,0.986793,0.984172,0.981319,$
		0.978235,0.974921,0.971380,0.965644,0.957216,0.947912,0.937751,0.926752,0.914937,$
		0.902328,0.888949,0.867491,0.836421,0.802741,0.766674,0.728460,0.688355,0.646650,0.603683]
g64_e34 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999985,0.999979,0.999973,0.999965,$
		0.999956,0.999946,0.999936,0.999924,0.999911,0.999897,0.999883,0.999859,0.999824,$
		0.999785,0.999742,0.999695,0.999645,0.999590,0.999531,0.999436,0.999296,0.999140,$
		0.998968,0.998781,0.998578,0.998360,0.998126,0.997747,0.997186,0.996563,0.995878,$
		0.995131,0.994323,0.993452,0.992521,0.991008,0.988777,0.986302,0.983584,0.980626,$
		0.977428,0.973993,0.970321,0.964376,0.955642,0.946003,0.935478,0.924089,0.911858,$
		0.898810,0.884970,0.862782,0.830679,0.795907,0.758708,0.719334,0.678066,0.635221,0.591172]
g64_e56 = [1.00000, 0.999999, 0.999997, 0.999994, 0.999990, 0.999985, 0.999979, 0.999972, 0.999963,$
		0.999954, 0.999944, 0.999933, 0.999921, 0.999908, 0.999894, 0.999878, 0.999854, 0.999817,$
		0.999777, 0.999732, 0.999684, 0.999631, 0.999574, 0.999514, 0.999415, 0.999269, 0.999107,$
		0.998929, 0.998735, 0.998524, 0.998298, 0.998055, 0.997661, 0.997079, 0.996432, 0.995721,$
		0.994946, 0.994107, 0.993204, 0.992237, 0.990667, 0.988351, 0.985783, 0.982963, 0.979893,$
		0.976576, 0.973012, 0.969203, 0.963037, 0.953980, 0.943987, 0.933079, 0.921279, 0.908611,$
		0.895101, 0.880776, 0.857823, 0.824636, 0.788723, 0.750340, 0.709761, 0.667289, 0.623272,0.578125]

g112_e12 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999986,0.999980,0.999974,0.999966,$
		0.999958,0.999948,0.999938,0.999927,0.999914,0.999901,0.999887,0.999872,0.999856,$
		0.999839,0.999821,0.999802,0.999783,0.999762,0.999740,0.999718,0.999694,0.999670,$
		0.999645,0.999618,0.999591,0.999563,0.999534,0.999488,0.999424,0.999357,0.999285,$
		0.999210,0.999131,0.999048,0.998962,0.998872,0.998778,0.998680,0.998578,0.998473,$
		0.998364,0.998252,0.998135,0.997954,0.997698,0.997428,0.997142,0.996842,0.996527,$
		0.996197,0.995852,0.995492,0.995117,0.994727,0.994323,0.993903,0.993469,0.993020,$
		0.992556,0.991832,0.990815,0.989739,0.988605,0.987412,0.986160,0.984850,0.983481,$
		0.982054,0.980570,0.979028,0.977428,0.975771,0.974057,0.972286,0.970459,0.967612,$
		0.963620,0.959406,0.954971,0.950319,0.945451,0.940371,0.935079,0.929579,0.923874,$
		0.917966,0.911858,0.905553,0.899054,0.892364,0.885487,0.874824,0.859980,0.844442,$
		0.828237,0.811394,0.793939,0.775902,0.757312,0.738201,0.718600,0.698543,0.678066,$
		0.657209,0.636012,0.614522,0.592790]
g112_e34 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999985,0.999979,0.999973,0.999965,$
		0.999956,0.999946,0.999936,0.999924,0.999911,0.999897,0.999883,0.999867,0.999851,$
		0.999833,0.999815,0.999795,0.999775,0.999753,0.999731,0.999707,0.999683,0.999658,$
		0.999631,0.999604,0.999576,0.999546,0.999516,0.999469,0.999403,0.999332,0.999258,$
		0.999180,0.999099,0.999013,0.998923,0.998829,0.998732,0.998631,0.998525,0.998416,$
		0.998303,0.998186,0.998066,0.997877,0.997612,0.997332,0.997036,0.996724,0.996397,$
		0.996055,0.995697,0.995324,0.994935,0.994531,0.994111,0.993676,0.993225,0.992759,$
		0.992278,0.991527,0.990473,0.989357,0.988181,0.986943,0.985645,0.984286,0.982867,$
		0.981388,0.979849,0.978250,0.976592,0.974874,0.973097,0.971261,0.969367,0.966416,$
		0.962278,0.957911,0.953317,0.948497,0.943454,0.938191,0.932711,0.927016,0.921109,$
		0.914993,0.908672,0.902147,0.895423,0.888503,0.881389,0.870364,0.855019,0.838964,$
		0.822227,0.804838,0.786826,0.768222,0.749058,0.729367,0.709184,0.688545,0.667489,$
		0.646061,0.624305,0.602274,0.580023]
g112_e56 = [1.00000,0.999999,0.999997,0.999994,0.999990,0.999985,0.999979,0.999972,0.999963,$
		0.999954,0.999944,0.999933,0.999921,0.999908,0.999894,0.999878,0.999862,0.999845,$
		0.999827,0.999807,0.999787,0.999766,0.999744,0.999720,0.999696,0.999671,0.999645,$
		0.999617,0.999589,0.999560,0.999529,0.999498,0.999449,0.999380,0.999307,0.999230,$
		0.999149,0.999064,0.998975,0.998882,0.998785,0.998684,0.998578,0.998469,0.998356,$
		0.998239,0.998117,0.997992,0.997796,0.997521,0.997230,0.996923,0.996600,0.996260,$
		0.995905,0.995533,0.995146,0.994742,0.994323,0.993887,0.993435,0.992968,0.992484,$
		0.991985,0.991206,0.990111,0.988954,0.987733,0.986448,0.985101,0.983691,0.982219,$
		0.980684,0.979087,0.977428,0.975708,0.973926,0.972082,0.970178,0.968213,0.965153,$
		0.960862,0.956333,0.951569,0.946572,0.941345,0.935891,0.930212,0.924312,0.918193,$
		0.911858,0.905312,0.898556,0.891595,0.884432,0.877071,0.865665,0.849795,0.833198,$
		0.815903,0.797943,0.779348,0.760153,0.740390,0.720096,0.699308,0.678066,0.656414,$
		0.634400,0.612073,0.589492,0.566719]


	if bins eq '36' then gainf = replicate(0.,36,7)
	if bins eq '64' then gainf = replicate(0.,64,7)
	if bins eq '112' then gainf = replicate(0.,112,7)
	

	;Determine parameters for each channel
	for bb=0,6 do begin
		case src[bb] of
	
			'E12AC': begin
						gain[bb] = cp.adc_gain_eac[0]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_eac[0]
						conversion[bb] = 1000./cp.boom_length[0]
						units[bb] = '(mV/m)!U2!N'
						zlims[bb,*] = [1d-3^2,1d-1^2]						
					 end
			'E34AC': begin
						gain[bb] = cp.adc_gain_eac[1]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_eac[1]
						conversion[bb] = 1000./cp.boom_length[1]
						units[bb] = '(mV/m)!U2!N'
						zlims[bb,*] = [1d-3^2,1d-1^2]
					 end
			'E56AC': begin
						gain[bb] = cp.adc_gain_eac[2]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_eac[2]
						conversion[bb] = 1000./cp.boom_length[2]
						units[bb] = '(mV/m)!U2!N'
						zlims[bb,*] = [1d-3^2,1d-1^2]
					 end
			'E12DC': begin
						gain[bb] = cp.adc_gain_edc[0]
						if bins eq '36' then gainf[*,bb] = g36_e12
						if bins eq '64' then gainf[*,bb] = g64_e12
						if bins eq '112' then gainf[*,bb] = g112_e12
						offset[bb] = cp.adc_offset_edc[0]
						conversion[bb] = 1000./cp.boom_length[0]
						units[bb] = '(mV/m)!U2!N'
						zlims[bb,*] = [1d-3^2,1d-1^2]
					 end
			'E34DC': begin
						gain[bb] = cp.adc_gain_edc[1]
						if bins eq '36' then gainf[*,bb] = g36_e34
						if bins eq '64' then gainf[*,bb] = g64_e34
						if bins eq '112' then gainf[*,bb] = g112_e34
						offset[bb] = cp.adc_offset_edc[1]
						conversion[bb] = 1000./cp.boom_length[1]
						units[bb] = '(mV/m)!U2!N'
						zlims[bb,*] = [1d-3^2,1d-1^2]
					 end
			'E56DC': begin
						gain[bb] = cp.adc_gain_edc[2]
						if bins eq '36' then gainf[*,bb] = g36_e56
						if bins eq '64' then gainf[*,bb] = g64_e56
						if bins eq '112' then gainf[*,bb] = g112_e56
						offset[bb] = cp.adc_offset_edc[2]
						conversion[bb] = 1000./cp.boom_length[2]
						units[bb] = '(mV/m)!U2!N'
						zlims[bb,*] = [1d-3^2,1d-1^2]
					 end
			'V1DC': begin
						gain[bb] = cp.adc_gain_vdc[0]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vdc[0]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V2DC': begin
						gain[bb] = cp.adc_gain_vdc[1]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vdc[1]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V3DC': begin
						gain[bb] = cp.adc_gain_vdc[2]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vdc[2]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V4DC': begin
						gain[bb] = cp.adc_gain_vdc[3]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vdc[3]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V5DC': begin
						gain[bb] = cp.adc_gain_vdc[4]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vdc[4]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V6DC': begin
						gain[bb] = cp.adc_gain_vdc[5]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vdc[5]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end

			'V1AC': begin
						gain[bb] = cp.adc_gain_vac[0]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vac[0]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V2AC': begin
						gain[bb] = cp.adc_gain_vac[1]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vac[1]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V3AC': begin
						gain[bb] = cp.adc_gain_vac[2]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vac[2]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V4AC': begin
						gain[bb] = cp.adc_gain_vac[3]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vac[3]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V5AC': begin
						gain[bb] = cp.adc_gain_vac[4]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vac[4]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end
			'V6AC': begin
						gain[bb] = cp.adc_gain_vac[5]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_vac[5]
						conversion[bb] = 1.
						units[bb] = 'volts!U2!N'
						zlims[bb,*] = [1d-4^2,1d-2^2]
					end

			'SCMU': begin
						gain[bb] = cp.adc_gain_msc[0]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_msc[0]
						conversion[bb] = pTf
						units[bb] = pTnT+'!U2!N'
						zlims[bb,*] = [(pTnT_zscale*1d-4)^2,(pTnT_zscale*1d-2)^2]
					end
			'SCMV': begin
						gain[bb] = cp.adc_gain_msc[1]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_msc[1]
						conversion[bb] = pTf
						units[bb] = pTnT+'!U2!N'
						zlims[bb,*] = [(pTnT_zscale*1d-4)^2,(pTnT_zscale*1d-2)^2]
					end
			'SCMW': begin
						gain[bb] = cp.adc_gain_msc[2]
						if bins eq '36' then gainf[*,bb] = replicate(1.,36)
						if bins eq '64' then gainf[*,bb] = replicate(1.,64)
						if bins eq '112' then gainf[*,bb] = replicate(1.,112)
						offset[bb] = cp.adc_offset_msc[2]
						conversion[bb] = pTf
						units[bb] = pTnT+'!U2!N'
						zlims[bb,*] = [(pTnT_zscale*1d-4)^2,(pTnT_zscale*1d-2)^2]
					end
		endcase
	endfor


		gain = gain/adc_factor  
	
		ylabel = src
		zlabel = 'RBSP'+probe+' EFW spec'
	
	
		ytitle = ylabel + '!C[Hz]'	
		ztitle = strarr(7)
		for j=0,6 do ztitle[j] = '!C' + units[j] + '/[Hz]!C!C' + zlabel
	
		for j=0,6 do begin
	
	
			get_data,tvar[j],data=data,dlimits=dlim
	
			if dlim.data_att.units eq 'ADC' then begin
	
			tst = size(data)
			
				;make sure there's data
				if tst[0] ne 0 then begin
		

					;subtract offset
					new_y = (double(data.y) - offset[j])

					;Compensate for 4 bit dynamic range compression
					new_y /= 4.^2
				
					;Divide the 0th bin by 4 (from weirdness doc)
					new_y[*,0] /= 4.

					;Compensate for using Hanning window
					new_y /= 0.375^2
				
					;Apply ADC conversion
					new_y *= adc_factor^2
				
					;Apply channel gain
					new_y *= gain[j]^2

					;Apply boom length to change from V to mV/m or from V to nT (or pT)		
					new_y *=  conversion[j]^2 
	
	
					;Apply freq dependent gain
					gtmp = reform(gainf[*,j])^2
					for qq=0,n_elements(new_y[*,0])-1 do new_y[qq,*] = new_y[qq,*]/gtmp
	
	
			
			
					new_data = {x:data.x,y:new_y,v:data.v}
					dlim.data_att.units = units[j]
					colors = [1]
				
					str_element, dlim, 'cdf', /delete
					str_element, dlim, 'code_id', /delete
				
					newname = tvar[j]
					store_data, newname,data=new_data,dlim=dlim
				
					options,newname,labflag=1
					options,newname,'ytitle',ytitle[j]
					options,newname,'ztitle',ztitle[j]
					ylim,newname,3,10000,1

		
				endif	
			endif else print,tvar[j], ' is already calibrated'
		endfor



	endif else begin
		print, ''
		dprint, 'Invalid datatype. Calibration aborted.'
		print, ''
		return
	endelse






	zlim,rbspx +'_efw_'+bins+'_spec0',zlims[0,0],zlims[0,1],1
	zlim,rbspx +'_efw_'+bins+'_spec1',zlims[1,0],zlims[1,1],1
	zlim,rbspx +'_efw_'+bins+'_spec2',zlims[2,0],zlims[2,1],1
	zlim,rbspx +'_efw_'+bins+'_spec3',zlims[3,0],zlims[3,1],1
	zlim,rbspx +'_efw_'+bins+'_spec4',zlims[4,0],zlims[4,1],1
	zlim,rbspx +'_efw_'+bins+'_spec5',zlims[5,0],zlims[5,1],1
	zlim,rbspx +'_efw_'+bins+'_spec6',zlims[6,0],zlims[6,1],1



	;--------------------------------------------
	;Fix the bin labels in the tplot structure
	;--------------------------------------------

	if bins eq '36' then begin

		tplot_names,'*efw_36_spec*',names=tnames
	
		fbins_36 = [findgen(8)*8,findgen(4)*16.+64.,$
				findgen(4)*32.+128.,findgen(4)*64.+256.,$
				findgen(4)*128.+512.,findgen(4)*256.+1024.,$
				findgen(4)*512.+2048.,findgen(4)*1024.+4096.]
	
		fcenter_36=(fbins_36[0:35] + fbins_36[1:35])/2.
	
		fbin_labels_36=strarr(36)
		fbin_labels_36[0:35]=string(fbins_36[0:35], format='(I0)')+'-'+$
					string(fbins_36[1:36], format='(I0)')+' Hz'
	
	
		for i=0,size(tnames,/n_elements)-1 do begin
			get_data,tnames[i],data=d,limits=l,dlimits=dl
			if is_struct(d) then d.v=fbins_36[0:35]
			store_data,tnames[i],data=d,limits=l,dlimits=dl
		endfor


	endif

	if bins eq '64' then begin

		tplot_names,'*efw_64_spec*',names=tnames
	
		fbins_64=[findgen(16)*8., findgen(8)*16.+128., $
					findgen(8)*32.+256, findgen(8)*64.+512.,$
					findgen(8)*128.+1024., findgen(8)*256.+2048., $
					findgen(9)*512.+4096.]
	
		fcenter_64=(fbins_64[0:63] + fbins_64[1:64])/2.
	
		fbin_labels_64=strarr(64)
		fbin_labels_64[0:63]=string(fbins_64[0:63], format='(I0)')+'-'+$
					string(fbins_64[1:64], format='(I0)')+' Hz'
	
	
		for i=0,size(tnames,/n_elements)-1 do begin
			get_data,tnames[i],data=d,limits=l,dlimits=dl
			if is_struct(d) then d.v=fbins_64[0:63]
			store_data,tnames[i],data=d,limits=l,dlimits=dl
		endfor
	endif


	if bins eq '112' then begin

		tplot_names,'*efw_112_spec*',names=tnames
	
		fbins_112 = [findgen(32)*8.,findgen(16)*16.+256.,$
				findgen(16)*32.+512.,findgen(16)*64.+1024.,$
				findgen(16)*128.+2048.,findgen(16)*256.+4096.]
	
		fcenter_112=(fbins_112[0:111] + fbins_112[1:112])/2.
	
		fbin_labels_112=strarr(112)
		fbin_labels_112[0:111]=string(fbins_112[0:111], format='(I0)')+'-'+$
					string(fbins_112[1:112], format='(I0)')+' Hz'
	
	
		for i=0,size(tnames,/n_elements)-1 do begin
			get_data,tnames[i],data=d,limits=l,dlimits=dl
			if is_struct(d) then d.v=fbins_112[0:111]
			store_data,tnames[i],data=d,limits=l,dlimits=dl
		endfor
	endif

end

